from csirtg_indicator import Indicator
import arrow
import json


def test_indicator_timestamps():
    f = arrow.utcnow().strftime("%Y-%m-%dT%H:%M:%S.%fZ")
    l = arrow.utcnow().strftime("%Y-%m-%dT%H:%M:%S.%fZ")
    r = arrow.utcnow().strftime("%Y-%m-%dT%H:%M:%S.%fZ")

    i = Indicator('192.168.1.1', firsttime=f, lasttime=l, reporttime=r)

    assert i.firsttime == arrow.get(f).datetime
    assert i.lasttime == arrow.get(l).datetime
    assert i.reporttime == arrow.get(r).datetime

    s = str(i)
    i = json.loads(s)

    assert i['firsttime'] == f
    assert i['lasttime'] == l
    assert i['reporttime'] == r


def test_indicator_timezones():
    t = '2017-03-06T11:41:48-06:00'
    a = arrow.get('2017-03-06T17:41:48Z').datetime

    i = Indicator('example.com', firsttime=t, lasttime=t, reporttime=t)

    assert i.firsttime == a
    assert i.lasttime == a
    assert i.reporttime == a


def test_lasttime_only():
    l = arrow.utcnow().strftime("%Y-%m-%dT%H:%M:%S.%fZ")

    i = Indicator('192.168.1.1', lasttime=l)

    assert i.lasttime == arrow.get(l).datetime

    s = str(i)
    i = json.loads(s)

    assert i.get('firsttime') is None


def test_firsttime_only():
    l = arrow.utcnow().strftime("%Y-%m-%dT%H:%M:%S.%fZ")

    i = Indicator('192.168.1.1', firsttime=l)

    assert i.firsttime == arrow.get(l).datetime

    s = str(i)
    i = json.loads(s)

    assert i.get('lasttime') is None



import json
from .plugin import Plugin
from csirtg_indicator import Indicator
from csirtg_indicator.constants import COLUMNS
from pprint import pprint


def _indicator_row(i, cols):
    if isinstance(i, Indicator):
        i = i.__dict__()

    r = dict()
    for c in cols:
        y = i.get(c, u'')
        if isinstance(y, list):
            if len(y) > 0 and isinstance(y[0], dict):
                y = ''
            else:
                y = u','.join(y)

        r[c] = y

    return r


def get_lines(data, cols=COLUMNS, stream=False):
    for i in data:

        i = _indicator_row(i, cols)

        if stream:
            i = [i]

        yield json.dumps(i)


class Json(Plugin):

    def __repr__(self):
        output = []
        
        for i in reversed(self.data):
            r = _indicator_row(i, self.cols)
                
            output.append(r)
            
        return json.dumps(output)



from .plugin import Plugin
import time
import os
from csirtg_indicator import Indicator

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

OUTPUT_PATH = os.getenv('CSIRTG_INDICATOR_BIND_PATH', '/etc/namedb')


def get_lines(data, filename=OUTPUT_PATH):
    output = StringIO()
    output.write("// generated by: {} at {}\n".format('csirtg-indicator', time.strftime('%Y-%m-%dT%H:%M:%S %Z')))

    for i in data:
        if isinstance(i, Indicator):
            i = i.__dict__()

        if i['itype'] != 'fqdn':
            continue

        output.write('zone "{}" {{type master; file "{}";}};'.format(i['indicator'], filename))
        yield output.getvalue()

        if isinstance(output, StringIO):
            output.truncate(0)


class Bind(Plugin):

    def __init__(self, *args, **kwargs):
        super(Bind, self).__init__(*args, **kwargs)

        self.output = kwargs.get('output', OUTPUT_PATH)

    def __repr__(self):
        text = [
            '// generated by: {} at {}'.format('csirtg-indicator', time.strftime('%Y-%m-%dT%H:%M:%S %Z'))
        ]
        for i in self.data:
            if isinstance(i, Indicator):
                i = i.__dict__()

            if i.get('itype') is not 'fqdn':
                pass

            text.append('zone "{}" {{type master; file "{}";}};'.format(i['indicator'], self.output))

        return '\n'.join(text)



import csv
from csirtg_indicator.constants import PYVERSION
from csirtg_indicator import Indicator
from csirtg_indicator.constants import COLUMNS

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

from .plugin import Plugin

if PYVERSION > 2:
    basestring = (str, bytes)


def get_lines(data, cols=COLUMNS, quoting=csv.QUOTE_ALL):
    output = StringIO()
    csvWriter = csv.DictWriter(output, cols, quoting=quoting)
    csvWriter.writeheader()

    for i in data:
        if isinstance(i, Indicator):
            i = i.__dict__()

        r = dict()
        for c in cols:
            y = i.get(c, u'')

            if type(y) is list:
                y = u','.join(y)

            if c == 'confidence' and y is None:
                y = 0.0

            if PYVERSION < 3:
                r[c] = y
                if isinstance(r[c], basestring):
                    r[c] = unicode(r[c]).replace('\n', r'\\n')
                    r[c] = r[c].encode('utf-8', 'ignore')
            else:
                r[c] = y
                if isinstance(r[c], basestring):
                    r[c] = r[c].replace('\n', r'\\n')

        csvWriter.writerow(r)
        yield output.getvalue().rstrip('\r\n')

        if isinstance(output, StringIO):
            output.truncate(0)


class Csv(Plugin):

    def __repr__(self):
        output = StringIO()
        
        csvWriter = csv.DictWriter(output, self.cols, quoting=csv.QUOTE_ALL)
        csvWriter.writeheader()

        for i in reversed(self.data):
            if isinstance(i, Indicator):
                i = i.__dict__()

            r = dict()
            for c in self.cols:
                y = i.get(c, u'')

                if isinstance(y, list):
                    if len(y) > 0 and isinstance(y[0], dict):
                        y = ''
                    else:
                        y = u','.join(y)

                if c == 'confidence' and y is None:
                    y = 0.0

                if PYVERSION < 3:
                    r[c] = y
                    if isinstance(r[c], basestring):
                        r[c] = unicode(r[c]).replace('\n', r'\\n')
                        r[c] = r[c].encode('utf-8', 'ignore')
                else:
                    r[c] = y
                    if isinstance(r[c], basestring):
                        r[c] = r[c].replace('\n', r'\\n')


            csvWriter.writerow(r)
        
        return output.getvalue().strip('\r\n')



from csirtg_indicator import Indicator
from csirtg_indicator.exceptions import InvalidIndicator


def _not(data):
    for d in data:
        try:
            d = Indicator(d)
            assert d.itype is not 'asn'
        except InvalidIndicator:
            pass


def test_asn_nok():
    data = [
        'example.com',
        'http://example.com:81',
        '192.168.1.1',
        '127.0.0./1'
    ]

    _not(data)


def test_asn_random():
    for d in [1, 10, 500, 65535]:
        assert Indicator(indicator='AS{}'.format(d)).itype == 'asn'
        print(Indicator(indicator='AS{}'.format(d)))



from .ztable import Table
from .zcsv import Csv
from .zjson import Json
from .zbro import Bro
from .zsnort import Snort
from .zbind import Bind
from .zbindrpz import BindRPZ

FORMATS = {
    'table': Table,
    'csv': Csv,
    'json': Json,
    'bro': Bro,
    'snort': Snort,
    'bind': Bind,
    'bindrpz': BindRPZ,
}

STIX_ENABLED = True

try:
    from stix.core import STIXHeader
    from .zstix import Stix
    FORMATS['stix'] = Stix
except ImportError:
    STIX_ENABLED = False



from csirtg_indicator import Indicator
import json
from csirtg_indicator.exceptions import InvalidIndicator
from random import randint, uniform

def test_indicator_ipv4():
    i = Indicator('192.168.1.1')
    assert i.is_private()
    assert i.indicator == '192.168.1.1'
    assert i.itype == 'ipv4'


def test_indicator_fqdn():
    i = Indicator('example.org')

    assert i.is_private() is False
    assert i.indicator == 'example.org'
    assert i.itype == 'fqdn'


def test_indicator_url():
    i = Indicator('http://example.org', tags='botnet,malware')

    assert i.is_private() is False
    assert i.indicator == 'http://example.org'
    assert i.itype is not 'fqdn'
    assert i.itype is 'url'
    assert 'botnet' in i.tags
    assert 'malware' in i.tags


def test_indicator_str():
    i = Indicator('http://example.org', tags='botnet,malware')

    s = json.loads(str(i))

    assert 'botnet' in s['tags']

    i = Indicator(**s)

    assert 'malware' in i.tags


def test_get_set():
    i = Indicator('localhost.com')

    try:
        i.indicator = 'localhost'
    except InvalidIndicator:
        pass

    i.indicator = 'localhost.org'
    assert i.itype == 'fqdn'

    i.indicator = 'https://192.168.1.1'
    assert i.itype == 'url'

    assert str(i)
    print(i)


def test_format_indicator():
    i = Indicator('example.com')
    i.altid = 'https://csirtg.io/search?q={indicator}'

    i = i.format_keys()
    assert i.altid == 'https://csirtg.io/search?q=example.com'


def test_indicator_dest():
    i = Indicator(indicator='192.168.1.1', dest='10.0.0.1', portlist="23", protocol="tcp", dest_portlist='21,22-23')
    assert i.dest
    assert i.dest_portlist


def test_confidence_str():
    c = uniform(0.001, 9.99)
    assert Indicator(indicator='192.168.1.1', confidence=c).confidence == c

    try:
        Indicator(indicator='192.168.1.1', confidence=',')
        raise RuntimeError('should not set confidence to ,')
    except ValueError:
        pass


def test_count_str():
    c = randint(0, 500)
    assert Indicator(indicator='192.168.1.1', count=c).count == c

    try:
        Indicator(indicator='192.168.1.1', count=',')
        raise RuntimeError('should not set confidence to ,')
    except ValueError:
        pass


def test_uuid():
    u1 = Indicator(indicator='192.168.1.1').uuid
    u2 = Indicator(indicator='192.168.1.1').uuid

    assert u1 is not None
    assert u2 is not None
    assert u1 != u2


def test_eq():
    u1 = Indicator(indicator='192.168.1.1')
    u2 = Indicator(indicator='192.168.1.1')

    u2.uuid = u1.uuid
    assert u1 == u2


# -*- coding: utf-8 -*-
import pytest
from csirtg_indicator.format.ztable import Table
from csirtg_indicator.indicator import Indicator


@pytest.fixture
def indicator():
    i = {
            'indicator': "example.com",
            'itype': 'fqdn',
            'provider': "me.com",
            'tlp': "amber",
            'confidence': "85",
            'reporttime': '2015-01-01T00:00:00Z',
            'asn_desc': u'telefÔnica brasil'
        }
    return Indicator(**i)


@pytest.fixture
def indicator_unicode(indicator):
    indicator.indicator = 'http://xz.job391.com/down/ï¿½ï¿½ï¿½ï¿½à¿ªï¿½ï¿½@89_1_60'
    return indicator


def test_format_table(indicator):

    print(Table([indicator]))
    assert Table([indicator])


def test_format_table_unicode(indicator_unicode):

    print(Table([indicator_unicode]))
    assert Table([indicator_unicode])

if __name__ == '__main__':
    test_format_table()



from .plugin import Plugin
import re
from csirtg_indicator import Indicator
from csirtg_indicator.constants import PYVERSION
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO


from pprint import pprint

itype = {
    'ipv4': 'ADDR',
    'ipv6': 'ADDR',
    'url': 'URL',
    'fqdn': 'DOMAIN',
    'md5': 'FILE_HASH',
    'sha1': 'FILE_HASH',
    'sha256': 'FILE_HASH',
}

COLUMNS = ['fields', 'indicator', 'indicator_type', 'meta.desc', 'meta.confidence', 'meta.source', 'meta.do_notice']

HEADER = '#' + '\t'.join(COLUMNS)
SEP = '|'


def _i_to_bro(i, cols):
    if isinstance(i, Indicator):
        i = i.__dict__()

    cols = ['indicator', 'itype', 'tags', 'confidence', 'provider']
    r = []
    
    if i['itype'] is 'url':
        i['indicator'] = re.sub(r'(https?\:\/\/)', '', i['indicator'])

    for c in cols:
        y = i.get(c, '-')

        if type(y) is list:
            y = SEP.join(y)

        if isinstance(y, int):
            y = str(y)

        if PYVERSION == 2:
            if isinstance(y, unicode):
                y = y.encode('utf-8')
        else:
            if isinstance(y, bytes):
                y = y.encode('utf-8')

        if c is 'itype':
            y = 'Intel::{0}'.format(itype[i[c]])

        r.append(str(y))

    # do_notice
    # https://www.bro.org/bro-exchange-2013/exercises/intel.html
    # https://github.com/csirtgadgets/massive-octo-spice/issues/438
    r.append('T')
    return "\t".join(r)


def get_lines(data, cols=COLUMNS):
    output = StringIO()
    output.write("{0}\n".format(HEADER))
    cols = ['indicator', 'itype', 'tags', 'confidence', 'provider']

    for i in data:
        i = _i_to_bro(i, cols)

        output.write(i)
        output.write("\n")
        yield output.getvalue()

        if isinstance(output, StringIO):
            output.truncate(0)


class Bro(Plugin):
    __name__ = 'bro'

    def __init__(self, *args, **kwargs):
        super(Bro, self).__init__(*args, **kwargs)

        self.cols = COLUMNS

    def __repr__(self):
        text = []
        for i in self.data:
            i = _i_to_bro(i, self.cols)
            text.append(i)

        text = "\n".join(text)

        text = "{0}\n{1}".format(HEADER, text)
        return text



from .plugin import Plugin
import time
import os
from csirtg_indicator import Indicator
from pprint import pprint

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

OUTPUT_PATH = os.getenv('CSIRTG_INDICATOR_BINDRPZ_PATH', '/etc/bind/rpz')


def get_lines(data, filename=OUTPUT_PATH):
    output = StringIO()
    text = [
        ";;; generated by: {} at {}".format('csirtg-indicator', time.strftime('%Y-%m-%dT%H:%M:%S %Z')),
        ';RPZ DATA!',
        '$TTL    1',
        '@       IN      SOA     localhost. root.localhost. (',
        '                     {}         ; Serial'.format(int(time.time())),
        '                         604800         ; Refresh',
        '                          86400         ; Retry',
        '                        2419200         ; Expire',
        '                          86400 )       ; Negative Cache TTL',
        ';',
        '@       IN      NS      localhost.'
    ]

    for t in text:
        output.write(t)
        yield str(output.getvalue())

        if isinstance(output, StringIO):
            output.truncate(0)

    for i in data:
        if isinstance(i, Indicator):
            i = i.__dict__()

        if i['itype'] != 'fqdn':
            continue

        output.write('{}        CNAME .'.format(i['indicator'], filename))
        output.write('*.{}        CNAME .'.format(i['indicator'], filename))
        yield output.getvalue()

        if isinstance(output, StringIO):
            output.truncate(0)


class BindRPZ(Plugin):

    def __init__(self, *args, **kwargs):
        super(BindRPZ, self).__init__(*args, **kwargs)

        self.output = kwargs.get('output', OUTPUT_PATH)

    def __repr__(self):
        text = [
            ';;; generated by: {} at {}'.format('csirtg-indicator', time.strftime('%Y-%m-%dT%H:%M:%S %Z')),
            '; RPZ DATA!',
            '$TTL    1',
            '@       IN      SOA     localhost. root.localhost. (',
            '                     {}         ; Serial'.format(int(time.time())),
            '                         604800         ; Refresh',
            '                          86400         ; Retry',
            '                        2419200         ; Expire',
            '                          86400 )       ; Negative Cache TTL',
            ';',
            '@       IN      NS      localhost.'
        ]

        for i in self.data:
            if isinstance(i, Indicator):
                i = i.__dict__()

            if i.get('itype') is not 'fqdn':
                pass

            text.append('{}        CNAME .'.format(i['indicator'], self.output))
            text.append('*.{}        CNAME .'.format(i['indicator'], self.output))

        return '\n'.join(text)



import arrow
from .plugin import Plugin
import stix.indicator
from stix.core import STIXPackage, STIXHeader
from cybox.common import Hash
from cybox.objects.file_object import File
from cybox.objects.address_object import Address
import re


class Stix(Plugin):

    def __init__(self, *args, **kwargs):
        super(Stix, self).__init__(*args, **kwargs)

    def _create_indicator(self, d):
        def _md5(keypair):
            shv = Hash()
            shv.simple_hash_value = keypair.get('indicator')

            f = File()
            h = Hash(shv, Hash.TYPE_MD5)
            f.add_hash(h)
            return f

        def _sha1(keypair):
            shv = Hash()
            shv.simple_hash_value = keypair.get('indicator')

            f = File()
            h = Hash(shv, Hash.TYPE_SHA1)
            f.add_hash(h)
            return f

        def _sha256(keypair):
            shv = Hash()
            shv.simple_hash_value = keypair.get('indicator')

            f = File()
            h = Hash(shv, Hash.TYPE_SHA256)
            f.add_hash(h)
            return f

        def _address_ipv4(address):
            if re.search('^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}', address):
                return 1

        def _address_fqdn(address):
            if re.search('^[a-zA-Z0-9.\-_]+\.[a-z]{2,6}$', address):
                return 1

        def _address_url(address):
            if re.search('^(ftp|https?):\/\/', address):
                return 1

        def _address(keypair):
            address = keypair.get('indicator')
            if _address_fqdn(address):
                return Address(address, 'fqdn')
            elif _address_ipv4(address):
                return Address(address, 'ipv4-addr')
            elif _address_url(address):
                return Address(address, 'url')

        indicator = stix.indicator.Indicator(timestamp=arrow.get(d.get('reporttime')).datetime)
        indicator.set_producer_identity(d.get('provider'))

        indicator.set_produced_time(arrow.utcnow().datetime)

        indicator.description = ','.join(d.get('tags'))

        itype = d.get('itype')

        if itype == 'md5':
            f = _md5(d)
        elif itype == 'sha1':
            f = _sha1(d)
        elif itype == 'sha256':
            f = _sha256(d)
        else:
            f = _address(d)

        indicator.add_object(f)
        return indicator

    def __repr__(self):
        stix_package = STIXPackage()
        stix_header = STIXHeader()
        stix_package.stix_header = stix_header

        for d in self.data:
            i = self._create_indicator(d)
            stix_package.add_indicator(i)

        return str(stix_package.to_xml())



from csirtg_indicator import Indicator
from csirtg_indicator.utils import is_subdomain
from faker import Faker
fake = Faker()


GOOD = [
    'hdxturkceizle.xn--6frz82g',
    'xn--1--blcfn0a0ai7a1a7e.xn--p1acf',
    'example.org',
    '1.2.3.4.com',
    'xn----jtbbmekqknepg3a.xn--p1ai',
    'dualstack.cddf-prod-frontend-1ho73vqwbi0tw-1326553765.us-east-1.elb.amazonaws.com',
    'laser-retargeting-server-production.us-east-1-prod-core-edge-public.spongecell.net',
]


def _not(data):
    for d in data:
        d = Indicator(d)
        assert d.itype != 'fqdn'


def test_fqdn_ip():
    data = ['192.168.1.0/24', '192.168.1.1', '2001:1608:10:147::21', '2001:4860:4860::8888']
    _not(data)


def test_fqdn_urls():
    data = [
        'http://192.168.1.1/1.html',
        'http://www41.xzmnt.com',
        'http://get.ahoybest.com/n/3.6.16/12205897/microsoft lync server 2010.exe',
        'https://example.com:443/1.html'
    ]
    _not(data)


def test_fqdn_ok():

    for d in GOOD:
        d = Indicator(d)
        assert d.itype is 'fqdn'


def test_fqdn_subdomain():
    data = [
        'www.yahoo.com',
        'www.ww2.yahoo.com',
    ]

    for d in data:
        print(Indicator(indicator=d).is_subdomain())
        assert Indicator(indicator=d).is_subdomain()


    data = [
        'yahoo.com',
        'google.com',
        'http://google.com',
        'https://www.google.com',
        'http://www2.www.google.com',
    ]

    for d in data:
        assert not Indicator(indicator=d).is_subdomain()


def test_fqdn_random():
    for d in range(0, 100):
        assert Indicator(indicator=fake.domain_name()).itype == 'fqdn'

